#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jun  1 15:11:33 2019

@author: Gabriele Coiana
"""
import numpy as np
from decomp import  read,  util#,  plot 
import sys

## =============================================================================
## Parameters
input_file = sys.argv[1]
a = read.read_parameters(input_file)[0]
Masses = read.read_parameters(input_file)[1]
n_atom_unit_cell = read.read_parameters(input_file)[2]
n_atom_conventional_cell = read.read_parameters(input_file)[3]
N1,N2,N3 = read.read_parameters(input_file)[4:7]
kinput = read.read_parameters(input_file)[7::][0]
file_eigenvectors = read.read_parameters(input_file)[8]
file_trajectory = read.read_parameters(input_file)[9]
file_initial_conf = read.read_parameters(input_file)[10]
system = read.read_parameters(input_file)[11]
DT = read.read_parameters(input_file)[12]
tau = read.read_parameters(input_file)[13]
T = read.read_parameters(input_file)[14]
windowing = read.read_parameters(input_file)[15]


tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
tot_atoms_conventional_cell = int(np.sum(n_atom_conventional_cell)) 
N1N2N3 = N1*N2*N3 # Number of cells
N = N1*N2*N3*tot_atoms_uc    # Number of atoms
masses, masses_for_animation = util.repeat_masses(Masses, n_atom_conventional_cell, n_atom_unit_cell, N1, N2, N3)
cH = 1.066*1e-6 # to [H]
cev = 2.902*1e-05 # to [ev]
kbH = 3.1668085639379003*1e-06# a.u. [H/K]
kbev = 8.617333262*1e-05 # [ev/K]
#### =============================================================================

print('\nHello, lets start!\n')
print('Getting input parameters...')
print(' System: ', system)
print(' Masses: ', Masses)
print(' Number of atoms per unit cell', n_atom_unit_cell)
print(' Supercell: ', N1, N2, N3)
print(' k path: ', [x.tolist() for x in kinput])
print(' Temperature: ', T, ' K')
print(' Number of timesteps chosen for correlation function: ', tau)
print(' Extent of timestep [ps]: ', DT*2.418884254*1e-05)
print()
print('Now calculating velocities...')

Nqpoints, qpoints_scaled, ks, freqs, eigvecs, distances, Hk = read.read_phonopy(file_eigenvectors, tot_atoms_uc)
Ruc, R0 = read.read_SPOSCAR(file_initial_conf, N1N2N3, N, n_atom_conventional_cell, n_atom_unit_cell)                       #rhombo or avg R0

Rt = np.loadtxt(file_trajectory)[:,1:]
Num_timesteps = int(len(Rt[:,0]))
print(' Number of timesteps of simulation: ', Num_timesteps, '\n')
tall = np.arange(Num_timesteps)*DT*2.418884254*1e-05 #conversion to picoseconds
dt_ps = tall[1]-tall[0]
Vt = np.diff(Rt,axis=0)/dt_ps*np.sqrt(masses)/np.sqrt(3*(N))


meta = util.max_freq(dt_ps, tau) #you want the max frequency plotted be 25 Thz
ZS, Zqs = np.zeros((meta,1+Nqpoints+1)), np.zeros((meta,tot_atoms_uc*3+1))


#this is for the total C(t)
t_tot, C_tot, freq_tot, G_tot = util.corr(tall,Vt,Vt,tau, ' ')
Z_tot = np.sqrt(np.conjugate(G_tot)*G_tot).real*cev/(kbev*T)
print(' Total spectrum, average total kinetic energy per dof: ', 1/2*C_tot[0]*cH, ' Hartree')
print(' Kinetic energy per dof according to eqp thm: ', 1/2*kbH*T, ' Hartree')
print('\t\t ratio: ', np.round((1/2*C_tot[0]*cH)/(1/2*kbH*T)*100,2), ' %\n')
ZS[:,0], ZS[:,1] = freq_tot[0:meta], Z_tot[0:meta]


print('Done. Performing decomposition...\n')

#anis = list(range(tot_atoms_uc*3))
#namedir = plot.create_folder(system)
#every_tot = int(tot_atoms_conventional_cell/tot_atoms_uc)
for i in range(Nqpoints):
    eigvec = eigvecs[i]
    freq_disp = freqs[i]
    k = ks[i]
    k_scal = qpoints_scaled[i]
    print('\t kpoint scaled: ', k_scal)
    print('\t kpoint:        ',np.round(k,3))
    
    #Creating the collective variable based on the k point
    Vcoll = np.zeros((Num_timesteps-1,tot_atoms_uc*3),dtype=complex)  
    for j,h,l in zip(range(tot_atoms_uc*3),np.repeat(range(0,N),3)*N1N2N3*3,np.tile(range(0,3),tot_atoms_uc)):
        vels = np.array(Vt[:,h+l:h+N1N2N3*3:3],dtype=complex)
        poss = R0[h:h+N1N2N3*3:3,:]
        x = np.multiply(vels,np.exp(-1j*np.dot(poss,k)))
        Vcoll[:,j] = np.sum(x,axis=1)
    Tkt = Vcoll  
    
##   this was to convert conventional into primitive
#    Tkt = np.zeros((Num_timesteps-1,tot_atoms_uc*3), dtype=complex)
#    for l,m in zip(range(0,tot_atoms_conventional_cell*3, every_tot*3),range(0,tot_atoms_uc*3,3)):
#        Tkt[:,m:m+3] = Vcoll[:,l:l+3]
    eigvecH = np.conjugate(eigvec.T)
    Qkt = np.dot(eigvecH,Tkt.T).T
    
    
    t, C, freq, G = util.corr(tall,Tkt,Tkt,tau, ' ')
    Z_q = np.sqrt(np.conjugate(G)*G).real*cev/(kbev*T)
    print('\t  kinetic energy of this kpoint: ',1/2*C.real[0]*cH, ' Hartree')
    print('\t  kinetic according to eqp thm: ',1/2*kbH*T, ' Hartree')
    print('\t  ratio: ', np.round((1/2*C.real[0]*cH)/(1/2*kbH*T)*100,2), ' %')
    ZS[:,i+2] = Z_q[0:meta]
        
    
    t_proj, C_proj, freq_proj, G_proj = util.corr(tall,Qkt,Qkt,tau, 'projected')
    Z = np.sqrt(np.conjugate(G_proj)*G_proj).real*cev/(kbev*T)
    
    Zqs[:,0] = Z_q[0:meta]
    Zqs[:,1:] = Z[0:meta,:]
    
    Params = np.zeros((3,tot_atoms_uc*3))
    for n in range(tot_atoms_uc*3):
        x_data, y_data = freq[0:meta], Z[0:meta,n]
        popt, perr = util.fit_to_lorentzian(x_data, y_data, k, n)
        print()
        print('\t\tFitting to Lorentzian, mode '+str(n)+'...')
        print('\t\tResonant frequency omega =',np.round(popt[0],2),' +-',np.round(perr[0],3))
        print('\t\tLinewidth gamma =',np.round(popt[2],2),' +-',np.round(perr[2],3))
        print()
        Params[:,n] = popt

#        anis[n] = plot.plot_with_ani(freq[0:meta],Z[0:meta,n],Z_q[0:meta], k, eigvec[:,n],freq_disp[n],n,Ruc,file_eigenvectors,masses_for_animation)
#        plot.save_proj(freq[0:meta],Z[0:meta,n],Z_q[0:meta], qpoints_scaled[i], Ruc, eigvec[:,n],freq_disp[n],n,namedir,masses_for_animation, popt, -1)  
   
    util.save_append('Zqs', k_scal.reshape(1,3), Zqs)
    util.save_append('quasiparticles', k_scal.reshape(1,3), Params)  
    print()


util.save('C(t)', np.vstack((t_tot,C_tot)).T)
util.save('ZS', ZS)
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 25 14:33:25 2019

@author: gc4217
"""
from scipy.optimize import curve_fit
import numpy as np
import os

def repeat_masses(Masses, n_atom_conventional_cell, n_atom_primitive_cell, N1, N2, N3):
    repeated_masses = np.array([])
    repeated_masses_for_ani = np.array([])
    for i in range(len(Masses)):
        mass = Masses[i]

        n = n_atom_conventional_cell[i]
        nprim = n_atom_primitive_cell[i]
        
        m = np.repeat(mass, N1*N2*N3*3*n)
        m_ani = np.repeat(mass,nprim*3)
        
        repeated_masses = np.concatenate((repeated_masses,m))
        repeated_masses_for_ani = np.concatenate((repeated_masses_for_ani,m_ani))
        
    masses = np.array(repeated_masses).flatten()
    masses_for_animation = np.array(repeated_masses_for_ani).flatten()
    
    return masses, masses_for_animation

def corr(tall,X,Y,tau,mode):
    M = len(tall)
    dt = tall[1] - tall[0]
    tmax = M - tau
    N = np.size(X[0]) 
    X0 = X[0:tmax,:]
    X2 = 1/tmax*np.sum(X[0:tmax,:]*X[0:tmax,:])
    C = []
    for n in range(tau):
        Xjj = Y[n:n+tmax,:]
        a = np.multiply(np.conjugate(X0),Xjj)
        b = 1/(tmax) * np.sum(a,axis=0)#/X2
        if (mode=='projected'):
            c = b
        else:
            c = np.sum(b)
        C.append(c)
    C = np.array(C)
    t = np.arange(0,tau)*dt
    freq = np.fft.fftfreq(tau,d=dt)
    Z = np.fft.fft(C,axis=0)
    return t, C, freq, Z

def lorentzian(x, x0, A, gamma):
    y = 1/np.pi *  A * 1/2*gamma / ((x - x0)**2 + (1/2*gamma)**2)
    return y

def save(filename, data):
    filename2 = filename
    if os.path.isfile(filename):
        n_of_files = len([name for name in os.listdir('.') if (os.path.isfile(name) and name==filename)])
        filename2 = filename+'_'+str(n_of_files)
        print(filename, ' already present. Saving it as ', filename2)
    np.savetxt(filename2,data) 
    return

def save_append(filename, data1, data2):
    filename2 = filename
#    if os.path.isfile(filename):
#        n_of_files = len([name for name in os.listdir('.') if (os.path.isfile(name) and name==filename)])
#        filename2 = filename+'_'+str(n_of_files)
#        print(filename, ' already present. Saving it as ', filename2)
        
    file = open(filename2,'ab')
    np.savetxt(file,data1)
    np.savetxt(file,data2)
    file.close()
    return

def max_freq(dt_ps, tau):
    #you want the max frequency plotted be 25 Thz
    max_freq = 0.5*1/dt_ps
    if (max_freq < 25):
        meta = int(tau/2)
    else:
        meta = int(tau/2*25/max_freq)
    return meta

def fit_to_lorentzian(x_data, y_data, k, n):
    if(n in [0,1,2] and np.allclose(k, [0,0,0])): #if acoustic modes at Gamma, don't fit anything
            popt, pcov = np.zeros(3), np.zeros((3,3))
    else:
        popt, pcov = curve_fit(lorentzian, x_data, y_data)
    perr = np.sqrt(np.diag(pcov))
    return popt, perr



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jan 24 14:40:14 2020

@author: gc4217
"""

import numpy as np
import os, sys
from decomp import plot, read


## =============================================================================
## Parameters
input_file = sys.argv[1]
mode = read.read_post_proc(input_file)[0]
n_atom_unit_cell = read.read_post_proc(input_file)[1]
file_eigenvectors = read.read_post_proc(input_file)[2]
max_Z = read.read_post_proc(input_file)[3]
kinput = read.read_post_proc(input_file)[4::][0]
labels = read.read_post_proc(input_file)[5]
modes = read.read_post_proc(input_file)[6]
temperatures_folders = read.read_post_proc(input_file)[7]



Ruc, masses_for_animation = 0,0 #atm you dont plot eigenvectors
tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
Nqpoints, qpoints_scaled, ks, freqs_disp, eigvecs, distances, Hk = read.read_phonopy(file_eigenvectors, tot_atoms_uc)
#### =============================================================================

if(mode==0):
    namedir = plot.create_folder('')
    freq = np.loadtxt('freq')
    meta = len(freq)
    for i in range(Nqpoints):
        to_skip = (1+meta)*i
        ZZ = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
        Z_q = ZZ[:,0]
        Z = ZZ[:,1:]
        to_skip2 = 4*i
        Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)

        k_scaled = qpoints_scaled[i]
        eigvec = eigvecs[i]
        freq_disp = freqs_disp[i]
        print(' Creating plots for k point = ', k_scaled)
        for n in range(tot_atoms_uc*3):
            params = Params[:,n]
            plot.save_proj(freq,Z[:,n],Z_q, qpoints_scaled[i], Ruc, eigvec[:,n],freq_disp[n],n,namedir,masses_for_animation, params,max_Z)


if(mode==1):
    freq = np.loadtxt('freq')
    meta = len(freq)
    ZQS = [np.zeros(meta)]
    count = 1
    accepted_qpoints = []
    accepted_labels = []
    for i in range(Nqpoints):
        to_skip = (1+meta)*i
        qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)
        for j in range(len(kinput)):
            if(np.allclose(qpoint, kinput[j]) and (qpoint.tolist() not in accepted_qpoints)):
                accepted_qpoints.append(qpoint.tolist())
                accepted_labels.append(labels[j])
                Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
                ZQS.append(Zq[:,0])
                count = count + 1
            
                #    Ztot = np.genfromtxt('ZS',usecols=0)
    ZQS = np.array(ZQS).T
    ZQS[:,0] = np.sum(ZQS[:,1::],axis=1)
    plot.plot1(freq,ZQS, accepted_labels)
    
    
if(mode==2): 
    freq = np.loadtxt('freq')
    meta = len(freq)
    ZQS = np.zeros((meta, tot_atoms_uc*3+1, len(kinput)))
    count = 0
    accepted_qpoints = []
    accepted_labels = []
    freqs_from_disp = np.zeros((tot_atoms_uc*3,len(kinput)))
    for i in range(Nqpoints):
        to_skip = (1+meta)*i
        qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)

        for j in range(len(kinput)):
            if(np.allclose(qpoint, kinput[j]) and (qpoint.tolist() not in accepted_qpoints)):
                accepted_qpoints.append(qpoint.tolist())
                accepted_labels.append(labels[j])

                for m in range(Nqpoints):
                    if(np.allclose(qpoint,qpoints_scaled[m])):
                        freqs_from_disp[:,count] = freqs_disp[m]
                        break
                
                
                Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
                ZQS[:,:,count] = Zq[:,:]
                count = count + 1
                
    
    plot.plot2(freq,ZQS, modes, accepted_labels, freqs_from_disp)
    
    
if(mode==3):
    subdirectories = [x[1] for x in os.walk(temperatures_folders)][0]
    subdirectories.sort(key= lambda x: float(x.strip('K')))
    Ts = [int(x.strip('K')) for x in subdirectories]
    frequencies = np.zeros((2,tot_atoms_uc*3,len(kinput),len(Ts)))
    count_T = 0
    for subdir in subdirectories:
        count_q = 0
        accepted_qpoints = []
        accepted_labels = []

        for i in range(Nqpoints):
            to_skip = (4)*i
            qpoint = np.genfromtxt(temperatures_folders+subdir+'/quasiparticles',skip_header=to_skip+0, max_rows=1)
            for j in range(len(kinput)):
                if(np.allclose(qpoint, kinput[j]) and (qpoint.tolist() not in accepted_qpoints)):
                    accepted_qpoints.append(qpoint.tolist())
                    accepted_labels.append(labels[j])
                    omegas_gammas = np.genfromtxt(temperatures_folders+subdir+'/quasiparticles',skip_header=to_skip+1, max_rows=3)
                    frequencies[:,:,count_q, count_T] = omegas_gammas[0::2,:]
                    count_q = count_q + 1
        count_T = count_T + 1
    
    diff =len(frequencies[0,0,:,0]) -  len(accepted_labels) 
    if(diff != 0): #this is to eliminate repetitive kpoints if present
        for h in range(diff):
            frequencies = np.delete(frequencies, -1, axis=2)
    plot.plot3(Ts, frequencies, modes, accepted_labels)

   
if(mode==4):
    # take the path the user inputs and find all the k points you can plot
    ks_path, x_labels, distances = read.read_path(kinput, Nqpoints, labels, Hk)
    frequencies = np.zeros((2,tot_atoms_uc*3,len(ks_path)))
    frequencies_disp = np.zeros((1,tot_atoms_uc*3,len(ks_path)))
    count = 0
    for i in range(Nqpoints):
        to_skip = (4)*i
        qpoint = np.genfromtxt('quasiparticles',skip_header=to_skip+0, max_rows=1)
        index_disp = np.min((np.argwhere(np.all(np.equal(qpoint, qpoints_scaled), axis=1))))
        if(np.any(np.all(np.equal(qpoint,ks_path), axis=1))): # e' un delirio questo ahaha, verifica che il qpoint trovato sia nel kpath
            params = np.genfromtxt('quasiparticles',skip_header=to_skip+1, max_rows=3)
            omegas_gammas = params[0::2,:]
            frequencies[:,:,count] = omegas_gammas
            frequencies_disp[:,:,count] = freqs_disp[index_disp,:]
            count = count + 1
            
        
    plot.plot4(distances, frequencies, frequencies_disp, ks_path, x_labels)
    

        
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    

